@@ -15,7 +15,7 @@ from PyQt5.QtGui import QIcon
 from PyQt5.QtWidgets import QApplication, QSplitter
 from PyQt5.QtWidgets import QDialog, QVBoxLayout, QPlainTextEdit, QTableWidgetItem
 
-from urh import constants
+from urh import settings
 from urh.util.Logger import logger
 
 PROJECT_PATH = None  # for referencing in external program calls
@@ -39,7 +39,7 @@ def profile(func):
 
 
 def set_icon_theme():
-    if sys.platform != "linux" or constants.SETTINGS.value("icon_theme_index", 0, int) == 0:
+    if sys.platform != "linux" or settings.read("icon_theme_index", 0, int) == 0:
         # noinspection PyUnresolvedReferences
         import urh.ui.xtra_icons_rc
         QIcon.setThemeName("oxy")
@@ -47,6 +47,15 @@ def set_icon_theme():
         QIcon.setThemeName("")
 
 
+def get_free_port():
+    import socket
+    s = socket.socket()
+    s.bind(("", 0))
+    port = s.getsockname()[1]
+    s.close()
+    return port
+
+
 def set_shared_library_path():
     shared_lib_dir = get_shared_library_path()
 
@@ -60,9 +69,14 @@ def set_shared_library_path():
             # LD_LIBRARY_PATH will not be considered at runtime so we explicitly load the .so's we need
             exts = [".so"] if sys.platform == "linux" else [".so", ".dylib"]
             import ctypes
+            libs = sorted(os.listdir(shared_lib_dir))
+            libusb = next((lib for lib in libs if "libusb" in lib), None)
+            if libusb:
+                # Ensure libusb is loaded first
+                libs.insert(0, libs.pop(libs.index(libusb)))
 
-            for lib in sorted(os.listdir(shared_lib_dir)):
-                if any(lib.endswith(ext) for ext in exts):
+            for lib in libs:
+                if lib.lower().startswith("lib") and any(ext in lib for ext in exts):
                     lib_path = os.path.join(shared_lib_dir, lib)
                     if os.path.isfile(lib_path):
                         try:
@@ -433,7 +447,7 @@ def validate_command(command: str):
 
 def set_splitter_stylesheet(splitter: QSplitter):
     splitter.setHandleWidth(4)
-    bgcolor = constants.BGCOLOR.lighter(150)
+    bgcolor = settings.BGCOLOR.lighter(150)
     r, g, b, a = bgcolor.red(), bgcolor.green(), bgcolor.blue(), bgcolor.alpha()
     splitter.setStyleSheet("QSplitter::handle:vertical {{margin: 4px 0px; "
                            "background-color: qlineargradient(x1:0, y1:0, x2:1, y2:0, "
@@ -447,3 +461,14 @@ def set_splitter_stylesheet(splitter: QSplitter):
                            "stop:0.5 rgba({0}, {1}, {2}, {3}),"
                            "stop:0.8 rgba(255, 255, 255, 0));"
                            "image: url(:/icons/icons/splitter_handle_vertical.svg);}}".format(r, g, b, a))
+
+
+def calc_x_y_scale(rect, parent):
+    view_rect = parent.view_rect() if hasattr(parent, "view_rect") else rect
+    parent_width = parent.width() if hasattr(parent, "width") else 750
+    parent_height = parent.height() if hasattr(parent, "height") else 300
+
+    scale_x = view_rect.width() / parent_width
+    scale_y = view_rect.height() / parent_height
+
+    return scale_x, scale_y
